@@ -2,7 +2,9 @@
 //! `ode-solvers` is a collection of numerical methods to solve ordinary differential equations (ODEs).
 
 // Re-export from external crate
-pub use crate::na::{DVector, SVector, Vector1, Vector2, Vector3, Vector4, Vector5, Vector6};
+pub use crate::na::{
+    DVector, OVector, SVector, Vector1, Vector2, Vector3, Vector4, Vector5, Vector6,
+};
 use nalgebra as na;
 
 // Declare modules
